﻿Imports SDKQBConnection
Public Class TestAppSDKQBConnection
    Public gAppName As String = "YourAppName"
    Public gCompanyAppName As String = "YourCompanyName"

    Public AppName As String = "TestApp"
    Public qb As New QBConnection
    Public gCompanyFile As New CompanyFile
    Public gConfig As New Config

    Private Sub TestAppSDKQBConnection_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        Try
            gConfig.Load()

            lvCompanyFiles.Items.Clear()
            Dim i As Integer = 0
            For Each tempCF As CompanyFile In gConfig.companyFiles.companyFiles
                lvCompanyFiles.Items.Add(tempCF.CompanyName)
                lvCompanyFiles.Items.Item(i).SubItems.Add(tempCF.CompanyPath)
            Next
        Catch ex As Exception
            MessageBox.Show(ex.Message, AppName, MessageBoxButtons.OK, MessageBoxIcon.Error)

        End Try
    End Sub

    Private Sub btnConnectQB_Click(sender As Object, e As EventArgs) Handles btnConnectQB.Click
        Try
            ConnectToQuickBooks("")
        Catch ex As Exception
            MessageBox.Show(ex.Message, AppName, MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try
    End Sub

    Private Sub btnCloseConnectionQB_Click(sender As Object, e As EventArgs) Handles btnCloseConnectionQB.Click
        Try
            CloseQBConnection()
        Catch ex As Exception
            MessageBox.Show(ex.Message, AppName, MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try
    End Sub

    Private Sub btnAdd_Click(sender As Object, e As EventArgs) Handles btnAdd.Click
        Try
            Dim proc = Process.GetProcessesByName("QBW")
            Dim proc32 = Process.GetProcessesByName("QBW32")

            If proc.Count = 0 And proc32.Count = 0 Then
                MessageBox.Show("Company file needs to be opened in QuickBooks before to be added.", gMyName, MessageBoxButtons.OK, MessageBoxIcon.Information)
                Exit Sub
            End If

            If proc.Count > 1 Then
                MessageBox.Show("When adding a company file, you can only have one QuickBooks instance opened.", gMyName, MessageBoxButtons.OK, MessageBoxIcon.Information)
                Exit Sub
            End If

            If ConnectToQuickBooks("") Then
                Dim companyFileOpened As String = qb.GetCompanyPath

                If Not gConfig.companyFiles.ItemExistsByCompanyPath(companyFileOpened) Is Nothing Then
                    MessageBox.Show("Currently open company file already exists.", AppName)
                    CloseQBConnection()
                    Exit Sub
                End If

                gCompanyFile.CompanyPath = companyFileOpened
                gCompanyFile.CompanyName = qb.GetCompanyName()

                CloseQBConnection()

                gConfig.companyFiles.companyFiles.Add(gCompanyFile)

                SaveConfiguration()

                lvCompanyFiles.Items.Clear()
                Dim i As Integer = 0
                For Each tempCF As CompanyFile In gConfig.companyFiles.companyFiles
                    lvCompanyFiles.Items.Add(tempCF.CompanyName)
                    lvCompanyFiles.Items.Item(i).SubItems.Add(tempCF.CompanyPath)
                Next

            End If
        Catch ex As Exception
            MessageBox.Show(ex.Message, AppName, MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try
    End Sub

    Public Function ConnectToQuickBooks(_CompanyPath As String) As Boolean
        Try
            Dim errorMessage As String = String.Empty

            Dim TrialOrLicenseKey As String = "The license key that you get on the email"

            qb.Connect(AppName, TrialOrLicenseKey, _CompanyPath, errorMessage)

            If errorMessage <> "" Then
                MessageBox.Show(errorMessage, AppName, MessageBoxButtons.OK, MessageBoxIcon.Error)
                Return False
            End If
            Return True
        Catch ex As Exception
            Return False
        End Try
    End Function

    Public Sub CloseQBConnection()
        Try
            qb.CloseConnection()

        Catch ex As Exception
            MessageBox.Show(ex.Message, AppName, MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try
    End Sub

    Public Sub SaveConfiguration()
        Try
            gConfig.Save()
            gConfig.Load()
        Catch ex As Exception
            MessageBox.Show(ex.Message, AppName, MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try
    End Sub

    Private Sub linkConfigFilePath_LinkClicked(sender As Object, e As LinkLabelLinkClickedEventArgs) Handles linkConfigFilePath.LinkClicked
        Try
            Dim _path As String = gConfig.configFilePathFolder
            Process.Start("explorer.exe", _path)
        Catch ex As Exception
            MessageBox.Show("Logs folder hasn't been set yet.", gMyName, MessageBoxButtons.OK, MessageBoxIcon.Information)
            glog.LogEntry(gError, ex.Message, AppName)
        End Try
    End Sub

    Private Sub btnDelete_Click(sender As Object, e As EventArgs) Handles btnDelete.Click
        Try
            If lvCompanyFiles.SelectedItems.Count = 0 Then
                MessageBox.Show("Select a company file", AppName, MessageBoxButtons.OK, MessageBoxIcon.Error)
                Exit Sub
            End If
            For x = 0 To lvCompanyFiles.SelectedItems.Count - 1

                Select Case MessageBox.Show("Are you sure you want to delete: " + lvCompanyFiles.SelectedItems(x).Text + "?", gMyName, MessageBoxButtons.YesNo, MessageBoxIcon.Warning)
                    Case MsgBoxResult.Yes

                        gConfig.companyFiles.Remove(gConfig.companyFiles.CompanyFileByName(lvCompanyFiles.SelectedItems(x).Text).CompanyName)
                        SaveConfiguration()
                        lvCompanyFiles.Items.RemoveAt(x)
                        lvCompanyFiles.Refresh()

                    Case MsgBoxResult.No
                        Exit Sub
                End Select
            Next
        Catch ex As Exception
            MessageBox.Show(ex.Message, AppName, MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try
    End Sub

    Private Sub btnCustomer_Click(sender As Object, e As EventArgs) Handles btnCustomerAdd.Click
        Try
            If lvCompanyFiles.SelectedItems.Count = 0 Then
                MessageBox.Show("Select a company file", AppName, MessageBoxButtons.OK, MessageBoxIcon.Error)
                Exit Sub
            End If

            Dim tempCustomers As New Customers
            Dim tempCustomer As New Customer

            If ConnectToQuickBooks(gConfig.companyFiles.CompanyFileByName(lvCompanyFiles.SelectedItems(0).Text).CompanyPath) = False Then
                Exit Sub
            End If

            'ADD NEW CUSTOMER AND JOB start
            tempCustomer = New Customer

            With tempCustomer
                .ListID = "123"
                .Name = "Joe Widgets 8"
                .IsActive = "True"
                .Operation = "Add"
            End With

            Dim tempCustomerJob As New Customer

            With tempCustomerJob
                .ListID = "1234"
                .Name = "Job 8"
                .ParentRef_FullName = tempCustomer.Name
                .Operation = "Add"
            End With

            Dim customersToBeSent As New List(Of Customer)

            customersToBeSent.Add(tempCustomer)
            customersToBeSent.Add(tempCustomerJob)

            tempCustomers.UpdateQuickBooks(Nothing, customersToBeSent)

            CloseQBConnection()

        Catch ex As Exception
            MessageBox.Show(ex.Message, AppName, MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try
    End Sub

    Private Sub btnCustomerGet_Click(sender As Object, e As EventArgs) Handles btnCustomerGet.Click
        Try
            If lvCompanyFiles.SelectedItems.Count = 0 Then
                MessageBox.Show("Select a company file", AppName, MessageBoxButtons.OK, MessageBoxIcon.Error)
                Exit Sub
            End If

            Cursor = Cursors.WaitCursor

            If ConnectToQuickBooks(gConfig.companyFiles.CompanyFileByName(lvCompanyFiles.SelectedItems(0).Text).CompanyPath) = False Then
                Exit Sub
            End If

            'GET CUSTOMERS first 20 

            Dim tempCustomers As New Customers
            Dim customersFromQuickBooks As New List(Of Customer)

            customersFromQuickBooks = tempCustomers.GetAllCustomer(Enumerations.qbActiveStatus.All, 20)

            Dim customerFullName As String = ""

            For Each tempCustomer As Customer In customersFromQuickBooks
                customerFullName += tempCustomer.FullName + Environment.NewLine
            Next

            Cursor = Cursors.Default

            MessageBox.Show(customerFullName, AppName, MessageBoxButtons.OK, MessageBoxIcon.Information)

            CloseQBConnection()

        Catch ex As Exception
            Cursor = Cursors.Default
            MessageBox.Show(ex.Message, AppName, MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try
    End Sub

    Private Sub btnInvoiceGet_Click(sender As Object, e As EventArgs) Handles btnInvoiceGet.Click
        Try
            If lvCompanyFiles.SelectedItems.Count = 0 Then
                MessageBox.Show("Select a company file", AppName, MessageBoxButtons.OK, MessageBoxIcon.Error)
                Exit Sub
            End If

            Cursor = Cursors.WaitCursor
            If ConnectToQuickBooks(gConfig.companyFiles.CompanyFileByName(lvCompanyFiles.SelectedItems(0).Text).CompanyPath) = False Then
                Exit Sub
            End If

            'GET Invoices and invoice lines, first 10 

            Dim tempInvoices As New Invoices
            Dim invoicesFromQuickBooks As New List(Of Invoice)

            invoicesFromQuickBooks = tempInvoices.GetAllInvoice(True, True, Enumerations.qbPaidStatus.All, 10)

            Dim invoiceRecords As String = ""

            For Each tempInvoice As Invoice In invoicesFromQuickBooks
                invoiceRecords += tempInvoice.TxnID + " " + tempInvoice.TxnNumber + " " + tempInvoice.CustomerRef_FullName + " " + tempInvoice.ClassRef_FullName + " " + tempInvoice.ARAccountRef_FullName + Environment.NewLine

                For Each tempInvoiceLineRet As InvoiceLineRet In tempInvoice.invoiceLineRetList
                    invoiceRecords += Space(4) + tempInvoiceLineRet.TxnLineID + " " + tempInvoiceLineRet.ItemRef_FullName + " " + tempInvoiceLineRet.Desc + " " + tempInvoiceLineRet.PARENT_IDKEY + Environment.NewLine
                Next
            Next
            Cursor = Cursors.Default

            MessageBox.Show(invoiceRecords, AppName, MessageBoxButtons.OK, MessageBoxIcon.Information)

            CloseQBConnection()
        Catch ex As Exception
            Cursor = Cursors.Default
            MessageBox.Show(ex.Message, AppName, MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try
    End Sub



    Private Sub btnInvoiceAdd_Click(sender As Object, e As EventArgs) Handles btnInvoiceAdd.Click
        Try
            If lvCompanyFiles.SelectedItems.Count = 0 Then
                MessageBox.Show("Select a company file", AppName, MessageBoxButtons.OK, MessageBoxIcon.Error)
                Exit Sub
            End If

            If ConnectToQuickBooks(gConfig.companyFiles.CompanyFileByName(lvCompanyFiles.SelectedItems(0).Text).CompanyPath) = False Then
                Exit Sub
            End If

            'Add Invoices and invoice lines

            Dim tempInvoices As New Invoices
            Dim tempInvoice As New Invoice
            Dim tempInvoiceLineRet As New InvoiceLineRet

            With tempInvoice
                .TxnID = "123456"
                .CustomerRef_FullName = "Abercrombie, Kristy"
                .TxnDate = "11/5/2021"
                .RefNumber = "111"
                .Operation = "Add"
            End With

            With tempInvoiceLineRet
                .ItemRef_FullName = "Installation"
                .Amount = "10"
                .SeqNum = 1
            End With
            tempInvoice.invoiceLineRetList.Add(tempInvoiceLineRet)

            tempInvoiceLineRet = New InvoiceLineRet

            With tempInvoiceLineRet
                .ItemRef_FullName = "Repairs"
                .Amount = "15"
                .SeqNum = 2
            End With
            tempInvoice.invoiceLineRetList.Add(tempInvoiceLineRet)

            Module_SDKCore.gSortRecordsBySeqNum = True

            Dim invoicesFromQuickBooks As New List(Of Invoice)

            invoicesFromQuickBooks = tempInvoices.UpdateQuickBooks(tempInvoice)

            Dim invoiceRecords As String = ""

            For Each tempInvoiceFromQB As Invoice In invoicesFromQuickBooks
                invoiceRecords += tempInvoiceFromQB.TxnID + " " + tempInvoiceFromQB.TxnNumber + " " + tempInvoiceFromQB.CustomerRef_FullName + " " + tempInvoiceFromQB.ClassRef_FullName + " " + tempInvoiceFromQB.ARAccountRef_FullName + Environment.NewLine

                For Each tempInvoiceLineRetFromQB As InvoiceLineRet In tempInvoiceFromQB.invoiceLineRetList
                    invoiceRecords += Space(4) + tempInvoiceLineRetFromQB.TxnLineID + " " + tempInvoiceLineRetFromQB.ItemRef_FullName + " " + tempInvoiceLineRetFromQB.Desc + " " + tempInvoiceLineRetFromQB.PARENT_IDKEY + Environment.NewLine
                Next
            Next

            MessageBox.Show(invoiceRecords, AppName, MessageBoxButtons.OK, MessageBoxIcon.Information)

            CloseQBConnection()
        Catch ex As Exception
            MessageBox.Show(ex.Message, AppName, MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try
    End Sub

    Private Sub btnTimeTrackingGet_Click(sender As Object, e As EventArgs) Handles btnTimeTrackingGet.Click
        Try
            If lvCompanyFiles.SelectedItems.Count = 0 Then
                MessageBox.Show("Select a company file", AppName, MessageBoxButtons.OK, MessageBoxIcon.Error)
                Exit Sub
            End If

            Cursor = Cursors.WaitCursor
            If ConnectToQuickBooks(gConfig.companyFiles.CompanyFileByName(lvCompanyFiles.SelectedItems(0).Text).CompanyPath) = False Then
                Exit Sub
            End If

            'GET TimeTracking, first 10 
            Dim tempTimeTrackings As New TimeTrackings
            Dim timeTrackingsFromQuickBooks As New List(Of TimeTracking)

            timeTrackingsFromQuickBooks = tempTimeTrackings.GetAllTimeTracking(10)

            Dim timeTrackingRecords As String = ""

            For Each tempTT As TimeTracking In timeTrackingsFromQuickBooks
                timeTrackingRecords += tempTT.TxnID + " " + tempTT.TxnDate + " " + tempTT.EntityRef_FullName + " " + tempTT.ItemServiceRef_FullName + " " + tempTT.Duration + Environment.NewLine
            Next
            Cursor = Cursors.Default

            MessageBox.Show(timeTrackingRecords, AppName, MessageBoxButtons.OK, MessageBoxIcon.Information)

            CloseQBConnection()
        Catch ex As Exception
            Cursor = Cursors.Default
            MessageBox.Show(ex.Message, AppName, MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try
    End Sub

    Private Sub btnTimeTrackingAdd_Click(sender As Object, e As EventArgs) Handles btnTimeTrackingAdd.Click
        Try
            If lvCompanyFiles.SelectedItems.Count = 0 Then
                MessageBox.Show("Select a company file", AppName, MessageBoxButtons.OK, MessageBoxIcon.Error)
                Exit Sub
            End If

            If ConnectToQuickBooks(gConfig.companyFiles.CompanyFileByName(lvCompanyFiles.SelectedItems(0).Text).CompanyPath) = False Then
                Exit Sub
            End If

            'Add TimeTracking

            Dim tempTimeTrackings As New TimeTrackings
            Dim tempTimeTracking As New TimeTracking
            Dim timeTrackingsFromQuickBooks As New List(Of TimeTracking)

            With tempTimeTracking
                .TxnID = "123456"
                .TxnDate = "11/8/2021"
                .EntityRef_FullName = "Dan T. Miller"
                .ItemServiceRef_FullName = "Installation"
                .Duration = "PT2H30M"
                .Operation = "Add"
            End With


            timeTrackingsFromQuickBooks = tempTimeTrackings.UpdateQuickBooks(tempTimeTracking)

            Dim timeTrackingRecords As String = ""

            For Each tempTT As TimeTracking In timeTrackingsFromQuickBooks
                timeTrackingRecords += tempTT.TxnID + " " + tempTT.TxnDate + " " + tempTT.EntityRef_FullName + " " + tempTT.ItemServiceRef_FullName + " " + tempTT.Duration + Environment.NewLine
            Next
            Cursor = Cursors.Default

            MessageBox.Show(timeTrackingRecords, AppName, MessageBoxButtons.OK, MessageBoxIcon.Information)

            CloseQBConnection()
        Catch ex As Exception
            MessageBox.Show(ex.Message, AppName, MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try
    End Sub
End Class